Package: ThurMod
Type: Package
Title: Thurstonian CFA and Thurstonian IRT Modeling
Version: 1.1.20
Date: 2026-05-04
Authors@R: person("Markus Thomas", "Jansen", email="mjansen@uni-wuppertal.de", role=c('aut','cre'), comment = c(ORCID = "0000-0002-5162-4409"))
Description: Fit Thurstonian forced-choice models (CFA (simple and factor) and IRT) in R. This package allows for the analysis of item response modeling (IRT) as well as confirmatory factor analysis (CFA) in the Thurstonian framework. Currently, estimation can be performed by 'Mplus' and 'lavaan'. References:
  Brown & Maydeu-Olivares (2011) <doi:10.1177/0013164410375112>;
  Jansen, M. T., & Schulze, R. (in review). The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data.;
  Maydeu-Olivares & Böckenholt (2005) <doi:10.1037/1082-989X.10.3.285>.
VignetteBuilder: knitr
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: stats, utils, mvtnorm, MASS, matrixStats, lavaan
Suggests: rmarkdown, knitr
URL: https://github.com/MarkusTJansen/ThurMod
BugReports: https://github.com/MarkusTJansen/ThurMod/issues
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.3
LazyData: true
Packaged: 2026-05-04 19:10:01 UTC; Jansen
Author: Markus Thomas Jansen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5162-4409>)
Maintainer: Markus Thomas Jansen <mjansen@uni-wuppertal.de>
Repository: CRAN
Date/Publication: 2026-05-04 19:50:02 UTC
Built: R 4.5.2; ; 2026-05-04 21:51:27 UTC; unix
