# Generated by roxygen2: do not edit by hand

export(blocksort)
export(count.combn)
export(count.xblocks)
export(designA)
export(fit.correct)
export(fit.lavaan)
export(fit.mplus)
export(get.scores)
export(get.xblocks)
export(get.xblocks.any)
export(i.name)
export(metablock)
export(mod.matrices)
export(pair.combn)
export(rankA)
export(read.mplus)
export(recode)
export(redundancies)
export(reliabiltyFS)
export(sim.data)
export(syntax.lavaan)
export(syntax.mplus)
